import datetime
import graphene
from graphql_jwt.decorators import login_required

from django.utils import timezone
from django.contrib.auth import get_user_model
from django.db.models import Sum

from allianceauth_pve.actions import EntryService
from allianceauth_pve.models import Rotation, EntryCharacter

from .inputs import EntryInput, CreateRotationInput, RotationCloseInput
from .types import RotationType, EntryType


class CreateRattingEntry(graphene.Mutation):
    class Arguments:
        input = EntryInput(required=True)
        rotation_id = graphene.Int(required=True)

    ok = graphene.Boolean()
    rotation = graphene.Field(RotationType)

    @login_required
    def mutate(root, info, input, rotation_id):
        entry = EntryService.create_entry(info.context.user, rotation_id, input.estimated_total, input.shares)
        return CreateRattingEntry(ok=True, rotation=entry.rotation)


class ModifyRattingEntry(graphene.Mutation):
    ok = graphene.Boolean()
    rotation = graphene.Field(RotationType)

    class Arguments:
        input = EntryInput(required=True)
        entry_id = graphene.ID(required=True)

    @login_required
    def mutate(root, info, input, entry_id):
        entry = EntryService.edit_entry(info.context.user, entry_id, input.estimated_total, input.shares)
        return ModifyRattingEntry(ok=True, rotation=entry.rotation)


class DeleteRattingEntry(graphene.Mutation):
    ok = graphene.Boolean()
    rotation = graphene.Field(RotationType)

    class Arguments:
        entry_id = graphene.ID(required=True)

    @login_required
    def mutate(root, info, entry_id):
        rotation = EntryService.delete_entry(info.context.user, entry_id)
        return DeleteRattingEntry(ok=True, rotation=rotation)


class CreateRotation(graphene.Mutation):
    ok = graphene.Boolean()
    rotation = graphene.Field(RotationType)

    class Arguments:
        input = CreateRotationInput(required=True)

    @login_required
    def mutate(root, info, input):
        user = info.context.user

        if not user.is_admin:
            raise Exception('Permission Denied')

        rotation = Rotation.objects.create(
            name=input.name,
            tax_rate=input.tax_rate,
            priority=input.priority if input.priority else 0,
        )

        return CreateRotation(ok=True, rotation=rotation)


class CloseRotation(graphene.Mutation):
    ok = graphene.Boolean()

    class Arguments:
        input = RotationCloseInput(required=True)

    @login_required
    def mutate(root, info, input):
        user = info.context.user
        rotation = Rotation.objects.get(pk=input.rotation_id)

        if not user.is_admin:
            raise Exception('Permission Denied')

        if rotation.is_closed:
            raise Exception('Rotation is closed and can not be modified')

        rotation.actual_total = input.sales_value
        rotation.is_closed = True
        rotation.closed_at = timezone.now()
        rotation.save()

        # update_rotation_totals(rotation_id=rotation.pk).execute()

        for entry in rotation.entries.all():
            entry.update_share_totals()

        shares = EntryCharacter.objects.filter(entry__rotation=rotation)
        User = get_user_model()
        chars = User.objects.filter(pk__in=shares.values('character'))
        rotation.summary.all().delete()

        for character in chars.all():
            shares = EntryCharacter.objects.filter(entry__rotation=rotation, character=character)

            shares_totals = shares.aggregate(estimated_total=Sum('estimated_share_total'), actual_total=Sum('actual_share_total'))

            helped_setups = 0

            start_date = rotation.created_at.date()
            end_date = rotation.closed_at.date() if rotation.is_closed else timezone.now().date()

            shares_helped_setup = shares.filter(helped_setup=True)

            for day in (start_date + datetime.timedelta(n) for n in range((end_date - start_date).days + 1)):
                for day_share in shares_helped_setup.filter(entry__created_at__date=day).all():
                    if day_share.entry.shares.count() > 2:
                        helped_setups += 1
                        break

            rotation.summary.update_or_create(
                character=character,
                defaults={
                    'estimated_total': shares_totals['estimated_total'],
                    'actual_total': shares_totals['actual_total'],
                    'helped_setup': helped_setups,
                }
            )

        return CloseRotation(ok=True)


class Mutation:
    allianceauth_pve_create_entry = CreateRattingEntry.Field()
    allianceauth_pve_modify_entry = ModifyRattingEntry.Field()
    allianceauth_pve_delete_entry = DeleteRattingEntry.Field()
    allianceauth_pve_create_rotation = CreateRotation.Field()
    allianceauth_pve_close_rotation = CloseRotation.Field()



import graphene
from graphql_jwt.decorators import login_required

from graphene_django_extras import DjangoFilterPaginateListField, LimitOffsetGraphqlPagination

from allianceauth_pve.models import Rotation

from .types import RotationType, EntryType


class Query:
    rotation = graphene.Field(RotationType, id=graphene.Int(required=True))
    rotations = graphene.List(RotationType)
    # running_averages = graphene.List(CharacterStatsType, start_date=graphene.Date(required=True), end_date=graphene.Date())
    # char_running_averages = graphene.Field(CharacterStatsType, start_date=graphene.Date(required=True), end_date=graphene.Date())
    active_rotations = graphene.List(RotationType)
    rotation_entries = DjangoFilterPaginateListField(EntryType, pagination=LimitOffsetGraphqlPagination(), fields=['rotation_id'])

    @login_required
    def resolve_rotation(self, info, id):
        return Rotation.objects.get(pk=id)

    @login_required
    def resolve_rotations(self, info):
        return Rotation.objects.filter(is_closed=True).order_by('-closed_at')

    # @login_required
    # def resolve_running_averages(self, info, start_date, end_date=now().date()):
    #     results = [
    #         {
    #             'character': char,
    #             **RattingService.character_stats_task(char.character_id, start_date.isoformat(), end_date.isoformat())
    #         } for char in CharacterService.get_all_instances()
    #     ]

    #     return sorted(results, key=lambda k: k['total_earned'], reverse=True)

    # @login_required
    # def resolve_char_running_averages(self, info, start_date, end_date=now().date()):
    #     char = info.context.user
    #     return {'character': char, **RattingService.character_stats_task(char.character_id, start_date.isoformat(), end_date.isoformat())}

    @login_required
    def resolve_active_rotations(self, info):
        return Rotation.objects.filter(is_closed=False).order_by('-priority')



import graphene
from graphql_jwt.decorators import login_required
from esi import app_settings
from requests_oauthlib import OAuth2Session
from django.conf import settings

from allianceauth.eveonline.models import EveCharacter

from .types import GroupType, UserProfileType
from ..eveonline.types import EveCharacterType

if 'allianceauth.eveonline.autogroups' in settings.INSTALLED_APPS:
    _has_auto_groups = True
    from allianceauth.eveonline.autogroups.models import *
else:
    _has_auto_groups = False


class Query:
    login_url = graphene.String()
    me = graphene.Field(UserProfileType)
    user_groups = graphene.List(GroupType)
    user_characters = graphene.List(EveCharacterType, description="List of the user's alts")

    def resolve_login_url(self, info):
        oauth = OAuth2Session(
            app_settings.ESI_SSO_CLIENT_ID,
            redirect_uri=app_settings.ESI_SSO_CALLBACK_URL,
            scope=getattr(settings, 'GRAPHQL_LOGIN_SCOPES', ['publicData'])
        )

        redirect_url, state = oauth.authorization_url(app_settings.ESI_OAUTH_LOGIN_URL)

        return redirect_url

    @login_required
    def resolve_me(self, info):
        return info.context.user.profile

    @login_required
    def resolve_user_groups(self, info):
        groups = info.context.user.groups.all()
        if _has_auto_groups:
            groups = groups\
                .filter(managedalliancegroup__isnull=True)\
                .filter(managedcorpgroup__isnull=True)
        return groups.order_by('name')

    @login_required
    def resolve_user_characters(self, info):
        return EveCharacter.objects.filter(character_ownership__user=info.context.user)\
            .select_related()\
            .order_by('character_name')



import graphene
from graphene_django import DjangoObjectType
from allianceauth.eveonline.models import EveCharacter, EveFactionInfo, EveAllianceInfo, EveCorporationInfo


class EveFactionInfoType(DjangoObjectType):
    class Meta:
        model = EveFactionInfo


class EveAllianceInfoType(DjangoObjectType):
    class Meta:
        model = EveAllianceInfo


class EveCorporationInfoType(DjangoObjectType):
    class Meta:
        model = EveCorporationInfo


class EveCharacterType(DjangoObjectType):
    is_biomassed = graphene.Boolean(required=True)
    alliance = graphene.Field(EveAllianceInfoType)
    corporation = graphene.Field(EveCorporationInfoType, required=True)
    faction = graphene.Field(EveFactionInfoType)

    class Meta:
        model = EveCharacter



from django.core import signing
from django.shortcuts import redirect, HttpResponse
from django.conf import settings
from django.contrib.auth.models import User

REGISTRATION_SALT = getattr(settings, "REGISTRATION_SALT", "registration")


def verify_email(request):
    activation_key = request.GET.get('activation_key')
    try:
        dump = signing.loads(activation_key, salt=REGISTRATION_SALT, max_age=settings.ACCOUNT_ACTIVATION_DAYS * 86400)
    except signing.BadSignature:
        return HttpResponse("Invalid signature")

    user = User.objects.get(pk=dump[0])
    user.email = dump[1]
    user.is_active = True
    user.save()

    site = getattr(settings, 'REDIRECT_SITE')
    path = getattr(settings, 'REDIRECT_PATH', '/registration/callback/')

    return redirect(site + path)



import graphene
import importlib
from django.conf import settings
from allianceauth.services.hooks import get_extension_logger

logger = get_extension_logger(__name__)

community_creations = [
    'allianceauth_pve',
]


def create_schema() -> graphene.Schema:
    mutations = []
    queries = []
    for app in settings.INSTALLED_APPS:
        if app.startswith('allianceauth.'):
            import_module = app.replace('allianceauth.', 'allianceauth_graphql.')
        elif app in community_creations:
            import_module = f'allianceauth_graphql.community_creations.{app}_integration'
        else:
            import_module = None

        if import_module is not None:
            try:
                module = importlib.import_module(import_module)
            except ModuleNotFoundError:
                logger.debug(f"Loading of {app}: fail")
            else:
                logger.debug(f"Loading of {app}: success")
                queries.append(module.Query)
                mutations.append(module.Mutation)

    class Query(*queries, graphene.ObjectType):
        pass

    class Mutation(*mutations, graphene.ObjectType):
        pass

    return graphene.Schema(query=Query, mutation=Mutation)


schema = create_schema()



import graphene
from graphene_django import DjangoObjectType

from allianceauth_pve.models import Rotation, EntryCharacter, Entry


class EntryCharacterType(DjangoObjectType):
    class Meta:
        model = EntryCharacter


class EntryType(DjangoObjectType):
    estimated_total_after_tax = graphene.Float()
    estimated_share_value = graphene.Float()
    total_shares_count = graphene.Int()

    class Meta:
        model = Entry


class RotationType(DjangoObjectType):
    sales_percentage = graphene.Float()
    days_since = graphene.Int()
    estimated_total = graphene.Float()

    class Meta:
        model = Rotation

    def resolve_entries(self, info):
        return self.entries.order_by('-created_at')

    def resolve_summary(self, info):
        return self.summary.order_by('-estimated_total')
